@@ -24,7 +24,12 @@ options(
   # lifecycle_verbosity="verbose",
   testthat.progress.max_fails = 50
 )
-crosstable_options(verbosity_autotesting="quiet")
+crosstable_reset_options(quiet=TRUE)
+crosstable_options(
+  verbosity_autotesting="quiet",
+  verbosity_na_cols="verbose",
+)
+# rlang::global_entrace()
 # prettycode::prettycode()
 
 
